package net.sf.sql2java;

import net.sf.sql2java.common.store.BasicProcessorStore;
import net.sf.sql2java.common.store.PackStore;
import net.sf.sql2java.common.store.ProcessorStore;
import net.sf.sql2java.conversion.Convertor;
import net.sf.sql2java.conversion.ConvertorConfigurer;
import net.sf.sql2java.fetching.sql.SQLFetcher;
import net.sf.sql2java.fetching.sql.SQLFetcherConfigurer;
import net.sf.sql2java.generator.Generator;
import net.sf.sql2java.generator.GeneratorConfigurer;
import net.sf.sql2java.generator.template.pack.TemplatePackConfigurer;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.log4j.Logger;

public class ApplicationContext {
	private static final Logger LOG = Logger.getLogger(ApplicationContext.class);
	
	protected ProcessorStore store;
	protected SQLFetcher fetcher;
	protected Generator generator;
	protected PackStore packStore;
	protected Convertor convertor;
	
	protected Configuration config;
	
// -- Thread-safe, lazy-loaded Singleton ---------------------------------------------------------------
	
	private static class ApplicationContextHolder {
		private static final ApplicationContext INSTANCE = new ApplicationContext();
	}
	
	public static ApplicationContext getInstance() {
		return ApplicationContextHolder.INSTANCE;
	}
	
	private ApplicationContext() {
		store = new BasicProcessorStore();
		fetcher = new SQLFetcher();
		generator = new Generator();
		packStore = new PackStore();
		convertor = new Convertor();
	}
	
// -- Configuration -----------------------------------------------------------
	
	public void configure() {
		SQLFetcherConfigurer fetcherConfigurer = new SQLFetcherConfigurer();
		GeneratorConfigurer generatorConfigurer = new GeneratorConfigurer();
		ConvertorConfigurer convertorConfigurer = new ConvertorConfigurer();
		TemplatePackConfigurer packConfigurer = new TemplatePackConfigurer();
		
		try {
			// -- configure the fetcher
			fetcherConfigurer.setConfiguration(config);
			fetcherConfigurer.configure(fetcher);
			
			// -- configure the convertor
			convertorConfigurer.setConfiguration(config);
			convertorConfigurer.configure(convertor);
			
			// -- configure the package manager
			packConfigurer.setConfiguration(config);
			packConfigurer.configure(packStore);

			// -- configure the generator
			generatorConfigurer.setConfiguration(config);
			generatorConfigurer.configure(generator);
		} catch (ConfigurationException ce) {
			LOG.error("Unable to load configuration!", ce);
		}
	}
	
// -- Getters -----------------------------------------------------------------
	
	public ProcessorStore getStore() {
		return store;
	}
	
	public SQLFetcher getFetcher() {
		return fetcher;
	}

	public Generator getGenerator() {
		return generator;
	}
	
	public PackStore getPackStore() {
		return packStore;
	}
	
	public Convertor getConvertor() {
		return convertor;
	}
	
	public Configuration getConfiguration() {
		return config;
	}
	
// -- Setters -----------------------------------------------------------------
	
	public void setFetcher(SQLFetcher fetcher) {
		this.fetcher = fetcher;
	}
	
	public void setConfiguration(Configuration config) {
		this.config = config;
		
		if (this.config != null)
			configure();
	}
	
	public void setStore(ProcessorStore store) {
		this.store = store;
	}
}
