package net.sf.sql2java.generator;

import java.util.Properties;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.log4j.Logger;

import net.sf.sql2java.common.configuration.ConfigurationHelper;
import net.sf.sql2java.common.configuration.Configurer;
import net.sf.sql2java.generator.merger.AbstractMerger;
import net.sf.sql2java.generator.template.pack.TemplatePackConfigurer;
import net.sf.sql2java.generator.writer.CodeWriter;
import net.sf.sql2java.utils.ClassUtils;

public class GeneratorConfigurer extends Configurer<Generator> {
	protected final Logger LOG = Logger.getLogger(GeneratorConfigurer.class);
	private TemplatePackConfigurer packageConfigurer;

// -- Constructors ------------------------------------------------------------
	
	public GeneratorConfigurer() {
		packageConfigurer = new TemplatePackConfigurer();
	}
	
// -- Configuration -----------------------------------------------------------
	
	@Override
	public void configure(Generator subject) throws ConfigurationException {
		// -- check if the configuration is valid
		if (getConfiguration() == null)
			throw new ConfigurationException("No configuration has been set");

		// -- read the packages
		packageConfigurer.configure(subject.getPackStore());
		
		// -- Code Writer
		CodeWriter writer = createWriter();
		subject.setWriter(writer);
		
		// -- Merger
		AbstractMerger merger = createMerger();
		subject.setMerger(merger);
	}
	
// -- Helpers -----------------------------------------------------------------
	
	protected CodeWriter createWriter() 
	throws ConfigurationException {
		// -- get the codewriter classname
		String classname = getConfiguration().getString("generator.writer[@class]");

		try {
			// -- get an instanceof of the classname
			Object obj = ClassUtils.executeConstructor(classname);
			
			// -- check if the codewriter is a codewriter
			if (!(obj instanceof CodeWriter))
				throw new ConfigurationException("Invalid writer class, It's not a CodeWriter");
			
			// -- convert the object to a writer
			CodeWriter writer = (CodeWriter) obj;
			
			// -- get the configuration
			Properties properties = ConfigurationHelper.readProperties(
					getConfiguration().subset("generator.writer.properties")
			);
			
			// -- set the properties on the result
			writer.setProperties(properties);
			
			// -- return the result object
			return writer;
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}
	
	protected AbstractMerger createMerger() 
	throws ConfigurationException {
		// -- get the merger classname
		String classname = getConfiguration().getString("generator.merger[@class]");

		try {
			// -- get an instanceof of the classname
			Object obj = ClassUtils.executeConstructor(classname);
			
			// -- check if the merger is a merger
			if (!(obj instanceof AbstractMerger))
				throw new ConfigurationException("Invalid merger class, It's not an AbstractMerger");
			
			// -- convert the object to a Merger
			AbstractMerger result = (AbstractMerger) obj;
			
			// -- get the configuration
			Properties properties = ConfigurationHelper.readProperties(
					getConfiguration().subset("generator.merger.properties")
			);
			
			// -- set the properties on the result
			result.setProperties(properties);
			
			// -- return the result object
			return result;
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}

// -- Setters -----------------------------------------------------------------
	
	@Override
	public void setConfiguration(Configuration configuration) {
		super.setConfiguration(configuration);
		
		packageConfigurer.setConfiguration(configuration);
	}
	
}
